(*----------------------------------------------------------------------------*
 *  Direct3D sample from DirectX 9.0 SDK Summer 2003                          *
 *  Delphi adaptation by Alexey Barkovoy (e-mail: clootie@ixbt.com)           *
 *                                                                            *
 *  Delphi versions 5-7 are supported                                         *
 *                                                                            *
 *  Modified: 28-Jan-2004                                                     *
 *                                                                            *
 *  Latest version can be downloaded from:                                    *
 *     http://clootie.narod.ru/delphi                                         *
 *----------------------------------------------------------------------------*)
//-----------------------------------------------------------------------------
// File: HLSLwithoutEffects.cpp
//
// Desc: Sample showing a simple vertex shader in D3DX High Level Shader
//       Language (HLSL) without using an ID3DXEffect interface.  Not using the 
//       ID3DXEffect interface is a more difficult method of using HLSL.  See
//       the BasicHLSL sample for a simpler method of using HLSL.
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------

unit HLSLWithoutEffectsUnit;

interface

uses
  Windows, Messages, Resource,
  Direct3D9, D3DX9, dxerr9, D3DApp, D3DEnumeration, D3DSettings,
  D3DFile, D3DFont, D3DUtil, DXUtil;

// C++Builder compile hack
{$NOINCLUDE Resource}
{$HPPEMIT '#include "Resource.hpp"'}

type
  //-----------------------------------------------------------------------------
  // Name: class CMyD3DApplication
  // Desc: Application class. The base class (CD3DApplication) provides the
  //       generic functionality needed in all Direct3D samples. CMyD3DApplication
  //       adds functionality specific to this sample program.
  //-----------------------------------------------------------------------------
  CMyD3DApplication = class(CD3DApplication)
  private
    // Font for drawing text
    m_pFont: ID3DXFont;

    // Scene
    m_pVB: IDirect3DVertexBuffer9;
    m_pIB: IDirect3DIndexBuffer9;
    m_dwNumVertices: DWORD;
    m_dwNumIndices: DWORD;
    m_pVertexShader: IDirect3DVertexShader9;
    m_pConstantTable: ID3DXConstantTable;
    m_pVertexDeclaration: IDirect3DVertexDeclaration9;
    m_dwSize: DWORD;
    m_bShowHelp: BOOL;

    // Navigation
    m_Camera: CModelViewerCamera;
  protected
    function OneTimeSceneInit: HResult; override;
    function InitDeviceObjects: HResult; override;
    function RestoreDeviceObjects: HResult; override;
    function InvalidateDeviceObjects: HResult; override;
    function DeleteDeviceObjects: HResult; override;
    function FinalCleanup: HResult; override;
    function Render: HResult; override;
    function FrameMove: HResult; override;
    function ConfirmDevice(const pCaps: TD3DCaps9; dwBehavior: DWORD;
      adapterFormat: TD3DFormat; backBufferFormat: TD3DFormat): HRESULT; override;

    function RenderText: HRESULT;
  public
    constructor Create;
    function MsgProc(hWnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; override;
  end;


const
  UnitName = 'HLSLWithoutEffectsUnit.pas';

implementation

//-----------------------------------------------------------------------------
// Name: CMyD3DApplication()
// Desc: Application constructor. Sets attributes for the app.
//-----------------------------------------------------------------------------
constructor CMyD3DApplication.Create;
begin
  inherited;
  m_Camera:= CModelViewerCamera.Create;

  m_strWindowTitle     := 'HLSLwithoutEffects';
  m_d3dEnumeration.AppUsesDepthBuffer := True;

  m_dwCreationWidth    := 640;
  m_dwCreationHeight   := 480;

  m_pFont              := nil;
  m_pIB                := nil;
  m_pVB                := nil;
  m_dwSize             := 64;
  m_dwNumIndices       := (m_dwSize - 1) * (m_dwSize - 1) * 6;
  m_dwNumVertices      := m_dwSize * m_dwSize;
  m_pVertexShader      := nil;
  m_pConstantTable     := nil;
  m_pVertexDeclaration := nil;

  m_bShowHelp          := True;
end;


//-----------------------------------------------------------------------------
// Name: OneTimeSceneInit()
// Desc: Paired with FinalCleanup().
//       The window has been created and the IDirect3D9 interface has been
//       created, but the device has not been created yet.  Here you can
//       perform application-related initialization and cleanup that does
//       not depend on a device.
//-----------------------------------------------------------------------------
function CMyD3DApplication.OneTimeSceneInit;
begin
  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Paired with DeleteDeviceObjects()
//       The device has been created.  Resources that are not lost on
//       Reset() can be created here -- resources in D3DPOOL_MANAGED,
//       D3DPOOL_SCRATCH, or D3DPOOL_SYSTEMMEM.  Image surfaces created via
//       CreateImageSurface are never lost and can be created here.  Vertex
//       shaders and pixel shaders can also be created here as they are not
//       lost on Reset().
//-----------------------------------------------------------------------------
var
  decl: array[0..1] of TD3DVertexElement9 =
  (
    (Stream: 0; Offset: 0; _Type: D3DDECLTYPE_FLOAT2; Method: D3DDECLMETHOD_DEFAULT; Usage: D3DDECLUSAGE_POSITION; UsageIndex: 0),
    ({D3DDECL_END})
  );
function CMyD3DApplication.InitDeviceObjects: HResult;
var
  hDC: Windows.HDC;
  nHeight: Integer;
  tszPath: array[0..511] of Char;
  pCode: ID3DXBuffer;
  dwShaderFlags: DWORD;
begin
  // Initialize the font
  hDC := GetDC(0);
  nHeight := -MulDiv( 9, GetDeviceCaps(hDC, LOGPIXELSY), 72);
  ReleaseDC(0, hDC);
  Result := D3DXCreateFont(m_pd3dDevice, nHeight, 0, FW_BOLD, 0, FALSE, DEFAULT_CHARSET,
              OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH or FF_DONTCARE,
              'Arial', m_pFont);
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('D3DXCreateFont', Result, UnitName);
    Exit;
  end;

  // Create vertex shader

  decl[1]:= D3DDECL_END;

  Result:= m_pd3dDevice.CreateVertexDeclaration(@decl, m_pVertexDeclaration);
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('CreateVertexDeclaration', Result, UnitName);
    Exit;
  end;

  // Find the vertex shader file
  Result:= DXUtil_FindMediaFileCb(tszPath, SizeOf(tszPath), 'HLSLwithoutEffects.vsh');
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('DXUtil_FindMediaFileCb', Result, UnitName);
    Exit;
  end;

  // Define DEBUG_VS and/or DEBUG_PS to debug vertex and/or pixel shaders with the shader debugger.
  // Debugging vertex shaders requires either REF or software vertex processing, and debugging
  // pixel shaders requires REF.  The D3DXSHADER_FORCE_*_SOFTWARE_NOOPT flag improves the debug
  // experience in the shader debugger.  It enables source level debugging, prevents instruction
  // reordering, prevents dead code elimination, and forces the compiler to compile against the next
  // higher available software target, which ensures that the unoptimized shaders do not exceed
  // the shader model limitations.  Setting these flags will cause slower rendering since the shaders
  // will be unoptimized and forced into software.  See the DirectX documentation for more information
  // about using the shader debugger.
  dwShaderFlags := 0;
  {$IFDEF DEBUG_VS}
    dwShaderFlags := dwShaderFlags or D3DXSHADER_FORCE_VS_SOFTWARE_NOOPT;
  {$ENDIF}
  {$IFDEF DEBUG_PS}
    dwShaderFlags := dwShaderFlags or D3DXSHADER_FORCE_PS_SOFTWARE_NOOPT;
  {$ENDIF}

  // Assemble the vertex shader from the file
  Result:= D3DXCompileShaderFromFile(tszPath, nil, nil, 'Ripple',
                                              'vs_1_1', dwShaderFlags, @pCode,
                                              nil, @m_pConstantTable);
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('D3DXCompileShaderFromFile', Result, UnitName);
    Exit;
  end;

  // Create the vertex shader
  Result := m_pd3dDevice.CreateVertexShader(PDWORD(pCode.GetBufferPointer), m_pVertexShader);
  pCode:= nil;
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('CreateVertexShader', Result, UnitName);
    Exit;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc: Paired with InvalidateDeviceObjects()
//       The device exists, but may have just been Reset().  Resources in
//       D3DPOOL_DEFAULT and any other device state that persists during
//       rendering should be set here.  Render states, matrices, textures,
//       etc., that don't change during rendering can be set once here to
//       avoid redundant state setting during Render() or FrameMove().
//-----------------------------------------------------------------------------
function CMyD3DApplication.RestoreDeviceObjects;
var
  pIndices: PWord;
  x, y: DWORD;
  pVertices: PD3DXVector2;
  fAspectRatio: Single;
begin
  // Restore font
  if Assigned(m_pFont) then m_pFont.OnResetDevice;

  // Setup render states
  m_pd3dDevice.SetRenderState(D3DRS_LIGHTING, iFalse);
  m_pd3dDevice.SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);

  // Create and initialize index buffer

  Result := m_pd3dDevice.CreateIndexBuffer(m_dwNumIndices * SizeOf(WORD),
                                           0, D3DFMT_INDEX16,
                                           D3DPOOL_DEFAULT, m_pIB, nil);
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('CreateIndexBuffer', Result, UnitName);
    Exit;
  end;

  Result := m_pIB.Lock(0, 0, Pointer(pIndices), 0);
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('Lock', Result, UnitName);
    Exit;
  end;

  for y := 1 to m_dwSize - 1 do
  begin
    for x := 1 to m_dwSize - 1 do
    begin
      pIndices^ := (y-1)*m_dwSize + (x-1); Inc(pIndices);
      pIndices^ := (y-0)*m_dwSize + (x-1); Inc(pIndices);
      pIndices^ := (y-1)*m_dwSize + (x-0); Inc(pIndices);

      pIndices^ := (y-1)*m_dwSize + (x-0); Inc(pIndices);
      pIndices^ := (y-0)*m_dwSize + (x-1); Inc(pIndices);
      pIndices^ := (y-0)*m_dwSize + (x-0); Inc(pIndices);
    end;
  end;

  Result := m_pIB.Unlock;
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('Unlock', Result, UnitName);
    Exit;
  end;

  // Create and initialize vertex buffer
  Result := m_pd3dDevice.CreateVertexBuffer(m_dwNumVertices * SizeOf(TD3DXVector2),
                                            0, 0,
                                            D3DPOOL_DEFAULT, m_pVB, nil);
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('CreateVertexBuffer', Result, UnitName);
    Exit;
  end;

  Result := m_pVB.Lock(0, 0, Pointer(pVertices), 0);
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('Lock', Result, UnitName);
    Exit;
  end;

  for y := 0 to m_dwSize - 1 do
  begin
    for x := 0 to m_dwSize - 1 do
    begin
      pVertices^ := D3DXVector2((x / (m_dwSize-1) - 0.5) * D3DX_PI,
                                (y / (m_dwSize-1) - 0.5) * D3DX_PI);
      Inc(pVertices);
    end;
  end;

  Result := m_pVB.Unlock;
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('Unlock', Result, UnitName);
    Exit;
  end;

  // Setup the camera
  fAspectRatio := m_d3dsdBackBuffer.Width / m_d3dsdBackBuffer.Height;
  m_Camera.SetProjParams(D3DX_PI / 3, fAspectRatio, 0.1, 100.0);
  m_Camera.SetWindow(m_d3dsdBackBuffer.Width, m_d3dsdBackBuffer.Height);
  m_Camera.SetViewQuat(D3DXQuaternion(-0.275, 0.3, 0.0, 0.9));

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FrameMove;
var
  m, mWorldViewProj, mWorld, mView, mProj: TD3DXMatrixA16;
begin
  // Update the camera
  m_Camera.FrameMove(m_fElapsedTime);

  // Set up the vertex shader constants
  mWorld := m_Camera.GetWorldMatrix^;
  mView  := m_Camera.GetViewMatrix^;
  mProj  := m_Camera.GetProjMatrix^;

  // mWorldViewProj := mWorld * mView * mProj;
  D3DXMatrixMultiply(m, mView, mProj);
  D3DXMatrixMultiply(mWorldViewProj, mWorld, m);

  m_pConstantTable.SetMatrix(m_pd3dDevice, 'mWorldViewProj', mWorldViewProj);
  m_pConstantTable.SetFloat(m_pd3dDevice, 'fTime', m_fTime);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.Render;
begin
  // Clear the backbuffer
  m_pd3dDevice.Clear(0, nil, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER,
                     $00003F3F, 1.0, 0);

  // Begin the scene
  if SUCCEEDED(m_pd3dDevice.BeginScene) then
  begin
    m_pd3dDevice.SetVertexDeclaration(m_pVertexDeclaration);
    m_pd3dDevice.SetVertexShader(m_pVertexShader);
    m_pd3dDevice.SetStreamSource(0, m_pVB, 0, SizeOf(TD3DXVector2));
    m_pd3dDevice.SetIndices(m_pIB);

    Result := m_pd3dDevice.DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, 0, m_dwNumVertices,
                                                0, m_dwNumIndices div 3);
    if FAILED(Result) then
    begin
      Result:= DXTRACE_ERR('DrawIndexedPrimitive', Result, UnitName);
      Exit;
    end;

    Result := RenderText;
    if FAILED(Result) then
    begin
      Result:= DXTRACE_ERR('RenderText', Result, UnitName);
      Exit;
    end;

    // End the scene.
    m_pd3dDevice.EndScene;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: RenderText()
// Desc: Draw the help & statistics for running sample
//-----------------------------------------------------------------------------
function CMyD3DApplication.RenderText: HRESULT;
 function DrawText(Font: ID3DXFont; pSprite: ID3DXSprite; pString: PChar; Count: Integer; pRect: PRect; Format: DWORD; Color: TD3DXColor): HRESULT;
 begin
   {$IFDEF UNICODE}
   Result:= Font.DrawTextW(pSprite, pString, Count, pRect, Format, D3DXColorToDWord(Color));
   {$ELSE}
   Result:= Font.DrawTextA(pSprite, pString, Count, pRect, Format, D3DXColorToDWord(Color));
   {$ENDIF}
   if FAILED(Result) then Result := DXTRACE_ERR('DrawText', Result, UnitName);
 end;
var
  rc: TRect;
  iLineY: Integer;
begin
  if (nil = m_pFont) then
  begin
    Result:= S_OK;
    Exit;
  end;

  // Output statistics
  iLineY := 0;
  SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
  Result:= DrawText(m_pFont, nil, m_strFrameStats, -1, @rc, DT_NOCLIP, D3DXColor(1.0, 1.0, 0.0, 1.0));
  if FAILED(Result) then Exit;

  SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
  Result := DrawText(m_pFont, nil, m_strDeviceStats, -1, @rc, DT_NOCLIP, D3DXColor(1.0, 1.0, 0.0, 1.0));
  if FAILED(Result) then Exit;

  if m_bShowHelp then
  begin
    iLineY := m_d3dsdBackBuffer.Height-15*6;
    SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
    Result := DrawText(m_pFont, nil, 'Controls:', -1, @rc, DT_NOCLIP, D3DXColor(1.0, 0.75, 0.0, 1.0));
    if FAILED(Result) then Exit;

    SetRect(rc, 20, iLineY, 0, 0);
    Result:= DrawText(m_pFont, nil, 'Rotate model: Left mouse button'#10 +
                                    'Rotate camera: Right mouse button'#10 +
                                    'Zoom camera: Mouse wheel scroll'#10 +
                                    'Hide help: F1'#10,
                                    -1, @rc, DT_NOCLIP, D3DXColor(1.0, 0.75, 0.0, 1.0));
    if FAILED(Result) then Exit;

    SetRect(rc, 250, iLineY, 0, 0);
    Result:= DrawText(m_pFont, nil, 'Change Device: F2'#10 +
                                    'View readme: F9'#10 +
                                    'Quit: ESC'#10,
                                     -1, @rc, DT_NOCLIP, D3DXColor(1.0, 0.75, 0.0, 1.0));
    if FAILED(Result) then Exit;
  end else
  begin
    SetRect(rc, 2, iLineY, 0, 0); // Inc(iLineY, 15);
    Result:= DrawText(m_pFont, nil, 'Press F1 for help',
                      -1, @rc, DT_NOCLIP, D3DXColor(1.0, 1.0, 1.0, 1.0));
    if FAILED(Result) then Exit;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc: Invalidates device objects.  Paired with RestoreDeviceObjects()
//-----------------------------------------------------------------------------
function CMyD3DApplication.InvalidateDeviceObjects;
begin
  if Assigned(m_pFont) then m_pFont.OnLostDevice;

  SAFE_RELEASE(m_pIB);
  SAFE_RELEASE(m_pVB);

  Result:= S_OK;
end;



//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Paired with InitDeviceObjects()
//       Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.  
//-----------------------------------------------------------------------------
function CMyD3DApplication.DeleteDeviceObjects;
begin
  SAFE_RELEASE(m_pFont);
  SAFE_RELEASE(m_pVertexShader);
  SAFE_RELEASE(m_pConstantTable);
  SAFE_RELEASE(m_pVertexDeclaration);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: FinalCleanup()
// Desc: Called before the app exits, this function gives the app the chance
//       to cleanup after itself.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FinalCleanup;
begin
  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: ConfirmDevice()
// Desc: Called during device initialization, this code checks the device
//       for some minimum set of capabilities.  In this sample we need
//       at least the support of vertex shader 1_0.
//-----------------------------------------------------------------------------
function CMyD3DApplication.ConfirmDevice(const pCaps: TD3DCaps9; dwBehavior: DWORD;
      adapterFormat: TD3DFormat; backBufferFormat: TD3DFormat): HRESULT;
begin
  // Debugging vertex shaders requires either REF or software vertex processing
  // and debugging pixel shaders requires REF.
  {$IFDEF DEBUG_VS}
    Result:= E_FAIL;
    if (pCaps.DeviceType <> D3DDEVTYPE_REF) and
       ((dwBehavior and D3DCREATE_SOFTWARE_VERTEXPROCESSING) = 0) then Exit;
  {$ENDIF}
  {$IFDEF DEBUG_PS}
    Result:= E_FAIL;
    if (pCaps.DeviceType <> D3DDEVTYPE_REF) then Exit;
  {$ENDIF}

  if( (dwBehavior and D3DCREATE_HARDWARE_VERTEXPROCESSING) or
      (dwBehavior and D3DCREATE_MIXED_VERTEXPROCESSING)) <> 0 then
  begin
    if (pCaps.VertexShaderVersion < D3DVS_VERSION(1,0)) then
    begin
      Result:= E_FAIL;
      Exit;
    end;
  end;

  Result:= S_OK;
end;




//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: Message proc function to handle key and menu input
//-----------------------------------------------------------------------------
function CMyD3DApplication.MsgProc(hWnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT;
begin
  // Handle camera movement
  m_Camera.HandleMessages(hWnd, uMsg, wParam, lParam);

  case uMsg of
    WM_COMMAND:
      case LOWORD(wParam) of
        IDM_TOGGLEHELP: m_bShowHelp := not m_bShowHelp;
      end;
  end;

  // Pass remaining messages to default handler
  Result:= inherited MsgProc(hWnd, uMsg, wParam, lParam);
end;

end.

